﻿namespace TestProject
{
    /// <summary>
    /// 달 위상 계산기
    /// </summary>
    public static class MoonPhaseCalculator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Enumeration
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 위상 - Phase

        /// <summary>
        /// 위상
        /// </summary>
        public enum Phase
        {
            /// <summary>
            /// 삭(신월)
            /// </summary>
            New,

            /// <summary>
            /// 초승달
            /// </summary>
            WaxingCrescent,

            /// <summary>
            /// 상현달
            /// </summary>
            FirstQuarter,

            /// <summary>
            /// 소망월
            /// </summary>
            WaxingGibbous,

            /// <summary>
            /// 보름달
            /// </summary>
            Full,

            /// <summary>
            /// 기망월
            /// </summary>
            WaningGibbous,

            /// <summary>
            /// 하현달
            /// </summary>
            LastQuarter,

            /// <summary>
            /// 그믐달
            /// </summary>
            WaningCrescent,
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 달 공전 일수
        /// </summary>
        private static double _moonCycleDayLength = 29.530588853;

        /// <summary>
        /// 참조 신월 날짜
        /// </summary>
        private static DateTime _referenceNewMoonDate = new DateTime(2017, 11, 18);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 위상 구하기 - GetPhase(date)

        /// <summary>
        /// 위상 구하기
        /// </summary>
        /// <param name="date">날짜</param>
        /// <returns>위상</returns>
        public static Phase GetPhase(DateTime date)
        {
            return GetPhase(GetPhaseValue(date));
        }

        #endregion
        #region 위상 텍스트 구하기 - GetPhaseText(phase)

        /// <summary>
        /// 위상 텍스트 구하기
        /// </summary>
        /// <param name="phase">위상</param>
        /// <returns>위상 텍스트</returns>
        public static string GetPhaseText(Phase phase)
        {
            switch(phase)
            {
                case Phase.New            : return "삭(신월)";
                case Phase.WaxingCrescent : return "초승달";
                case Phase.FirstQuarter   : return "상현달";
                case Phase.WaxingGibbous  : return "소망월";
                case Phase.Full           : return "보름달";
                case Phase.WaningGibbous  : return "기망월";
                case Phase.LastQuarter    : return "하현달";
                case Phase.WaningCrescent : return "그믐달";
                default                   : return string.Empty;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Public

        #region 위상 값 구하기 - GetPhaseValue(date)

        /// <summary>
        /// 위상 값 구하기
        /// </summary>
        /// <param name="date">날짜</param>
        /// <returns>위상 값</returns>
        private static double GetPhaseValue(DateTime date)
        {
            double phaseValue = (date - _referenceNewMoonDate).TotalDays;

            return phaseValue % _moonCycleDayLength;
        }

        #endregion
        #region 위상 구하기 - GetPhase(phaseValue)

        /// <summary>
        /// 위상 구하기
        /// </summary>
        /// <param name="phaseValue">위상 값</param>
        /// <returns>위상</returns>
        private static Phase GetPhase(double phaseValue)
        {
            if(phaseValue < 1 ) return Phase.New;
            if(phaseValue < 7 ) return Phase.WaxingCrescent;
            if(phaseValue < 8 ) return Phase.FirstQuarter;
            if(phaseValue < 14) return Phase.WaxingGibbous;
            if(phaseValue < 15) return Phase.Full;
            if(phaseValue < 22) return Phase.WaningGibbous;
            if(phaseValue < 23) return Phase.LastQuarter;
            if(phaseValue < 29) return Phase.WaningCrescent;

            return Phase.New;
        }

        #endregion
    }
}