﻿using System.Text.Json;

namespace TestProject
{
    /// <summary>
    /// 일출 서비스
    /// </summary>
    public class SunriseService
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Class
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 일출/일몰 결과 - SunriseSunsetResults

        /// <summary>
        /// 일출/일몰 결과
        /// </summary>
        private class SunriseSunsetResults
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Property
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 일출 - Sunrise

            /// <summary>
            /// 일출
            /// </summary>
            public string Sunrise { get; set; }

            #endregion
            #region 일몰 - Sunset

            /// <summary>
            /// 일몰
            /// </summary>
            public string Sunset { get; set; }

            #endregion
        }

        #endregion
        #region 일출/일몰 데이터 - SunriseSunsetData

        /// <summary>
        /// 일출/일몰 데이터
        /// </summary>
        private class SunriseSunsetData
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Property
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 결과 - Results

            /// <summary>
            /// 결과
            /// </summary>
            public SunriseSunsetResults Results { get; set; }

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field
 
        /// <summary>
        /// 서비스 URL
        /// </summary>
        private const string SERVICE_URL = "https://api.sunrise-sunset.org";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 데이터 시간 구하기 - GetData(latitude, longitude)

        /// <summary>
        /// 데이터 구하기
        /// </summary>
        /// <param name="latitude">위도</param>
        /// <param name="longitude">경도</param>
        /// <returns>(일출 시간, 일몰 시간)</returns>
        public async Task<(DateTime SunriseTime, DateTime SunsetTime)> GetData(double latitude, double longitude)
        {
            string url = $"{SERVICE_URL}/json?lat={latitude}&lng={longitude}&date=today";

            HttpClient client = new HttpClient();

            client.DefaultRequestHeaders.Add("Accept", "application/json");

            string json = await client.GetStringAsync(url);

            JsonSerializerOptions options = new JsonSerializerOptions { PropertyNameCaseInsensitive = true };

            SunriseSunsetData data = JsonSerializer.Deserialize<SunriseSunsetData>(json, options);

            return (DateTime.Parse(data.Results.Sunrise), DateTime.Parse(data.Results.Sunset));
        }

        #endregion
    }
}