﻿namespace TestProject;

/// <summary>
/// 달 위상 페이지
/// </summary>
public partial class MoonPhasePage : ContentPage
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 달 위상 딕셔너리
    /// </summary>
    private static Dictionary<MoonPhaseCalculator.Phase, string> _moonPhaseDictionary = new Dictionary<MoonPhaseCalculator.Phase, string>
    {
        { MoonPhaseCalculator.Phase.New           , "🌑" },
        { MoonPhaseCalculator.Phase.WaxingCrescent, "🌒" },
        { MoonPhaseCalculator.Phase.FirstQuarter  , "🌓" },
        { MoonPhaseCalculator.Phase.WaxingGibbous , "🌔" },
        { MoonPhaseCalculator.Phase.Full          , "🌕" },
        { MoonPhaseCalculator.Phase.WaningGibbous , "🌖" },
        { MoonPhaseCalculator.Phase.LastQuarter   , "🌗" },
        { MoonPhaseCalculator.Phase.WaningCrescent, "🌘" }
    };

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - MoonPhasePage()

    /// <summary>
    /// 생성자
    /// </summary>
    public MoonPhasePage()
    {
        InitializeComponent();

        MoonPhaseCalculator.Phase phase = MoonPhaseCalculator.GetPhase(DateTime.Now);

        this.dateLabel.Text          = DateTime.Today.ToString("yyyy/MM/dd");
        this.moonPhaseIconLabel.Text = _moonPhaseDictionary[phase];
        this.moonPhaseTextLabel.Text = MoonPhaseCalculator.GetPhaseText(phase);

        SetMoonPhaseLabels(this.phaseIconLabel1, this.phaseTextLabel1, 1);
        SetMoonPhaseLabels(this.phaseIconLabel2, this.phaseTextLabel2, 2);
        SetMoonPhaseLabels(this.phaseIconLabel3, this.phaseTextLabel3, 3);
        SetMoonPhaseLabels(this.phaseIconLabel4, this.phaseTextLabel4, 4);
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 달 위상 레이블 설정하기 - SetMoonPhaseLabels(phaseIconLabel, phaseTextLabel, dayOffset)

    /// <summary>
    /// 달 위상 레이블 설정하기
    /// </summary>
    /// <param name="phaseIconLabel">위상 아이콘 레이블</param>
    /// <param name="phaseTextLabel">위상 텍스트 레이블</param>
    /// <param name="dayOffset">날짜 오프셋</param>
    private void SetMoonPhaseLabels(Label phaseIconLabel, Label phaseTextLabel, int dayOffset)
    {
        MoonPhaseCalculator.Phase phase = MoonPhaseCalculator.GetPhase(DateTime.Now.AddDays(dayOffset));

        phaseIconLabel.Text = _moonPhaseDictionary[phase];
        phaseTextLabel.Text = DateTime.Now.AddDays(dayOffset).ToString("MM/dd");
    }

    #endregion
}