﻿namespace TestProject;

/// <summary>
/// 일출 페이지
/// </summary>
public partial class SunrisePage : ContentPage
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 위치 서비스 - LocationService

    /// <summary>
    /// 위치 서비스
    /// </summary>
    private ILocationService LocationService { get; set; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - SunrisePage()

    /// <summary>
    /// 생성자
    /// </summary>
    public SunrisePage()
    {
        InitializeComponent();

        LocationService = new LocationService();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region 표시시 처리하기 - OnAppearing()

    /// <summary>
    /// 표시시 처리하기
    /// </summary>
    protected override async void OnAppearing()
    {
        base.OnAppearing();

        this.activityIndicator.IsRunning = true;

        (DateTime SunriseTime, DateTime SunsetTime, TimeSpan DaylightTime) data = await GetData();

        this.dateLabel.Text         = DateTime.Today.ToString("yyyy-MM-dd");
        this.sunriseTimeLabel.Text  = data.SunriseTime.ToString("HH:mm");
        this.daylightTimeLabel.Text = $"{data.DaylightTime.Hours} 시간 {data.DaylightTime.Minutes} 분";
        this.sunsetTimeLabel.Text   = data.SunsetTime.ToString("HH:mm");

        this.activityIndicator.IsRunning = false;
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 데이터 구하기 - GetData()

    /// <summary>
    /// 데이터 구하기
    /// </summary>
    /// <returns>(일출 시간, 일몰 시간, 일광 시간)</returns>
    private async Task<(DateTime SunriseTime, DateTime SunsetTime, TimeSpan DaylightTime)> GetData()
    {
        (double Latitude, double Longitude) location = await LocationService.GetLocation();

        (DateTime SunriseTime, DateTime SunsetTime) data = await new SunriseService().GetData(location.Latitude, location.Longitude);

        DateTime sunriseTime  = data.SunriseTime.ToLocalTime();
        DateTime sunsetTime   = data.SunsetTime.ToLocalTime();
        TimeSpan daylightTime = sunsetTime.TimeOfDay - sunriseTime.TimeOfDay;

        return (sunriseTime, sunsetTime, daylightTime);
    }

    #endregion
}